package ACL2::Orden::Fabrica;
use ACL2::Orden;
use ACL2::AnalisisLisp;

#
#    Fbrica de rdenes
#    Copyright (C) 2006-2008 Antonio Garca Domnguez
#    $Id: Fabrica.pm 202 2008-06-27 22:47:53Z bluezio $
#
#    This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;


=head2 crear

  Uso: Mtodo pblico esttico.
  Responsabilidades: 
    Crea el tipo correcto de Orden para una
    S-expresin y salida dadas.
  Parmetros:

=over 2

=item *

orden: S-expresin con la orden completa para ACL2.

=item *

salida: Salida completa de ACL2.

=item *

paquete: Paquete Lisp al que pertenece la frmula.

=back

  Devuelve: 
    Descendiente de ACL2::Orden correspondiente, ya inicializado.
  Errores:
    Fallo fatal si la clase no existe.

=cut

sub crear
{
    my ($proto, $lisp, $salida, $paquete) = @_;

    # Ponemos en maysculas el enunciado del teorema, tal y como lo hace ACL2
    $lisp = uc $lisp;

    # Descomponemos la lista Lisp que representa al comando
    my @arbolLisp = ACL2::AnalisisLisp->dividir_lista($lisp);
    # Retiramos caracteres extraos del nombre de la orden
    $arbolLisp[0] =~ s/\W//g;
    my $clase     = "ACL2::Orden::" . ucfirst lc $arbolLisp[0];

    # Comprobamos que la clase existe
    eval "require($clase)";

    # A partir de ahora, con escrbir Defun.pm trataremos los defun,
    # Defthm.pm los defthm, y as sucesivamene.

    my $orden = bless { str_lisp => \$lisp,
			lisp   => \@arbolLisp, 
			salida => \$salida,
			paquete => $paquete}, $clase;

    # Hacemos que se inicialice
    $orden->inicializar;

    return $orden;
}

1;
